;+
;PROCEDURE:	load_wi_at 
;PURPOSE:	
;   loads WIND 3D Plasma Experiment orbit data for "tplot".
;INPUTS:
;  none, but will call "timespan" if time_range is not already set.
;KEYWORDS:
;  DATA:        Raw data can be returned through this named variable.
;  TIME_RANGE:  2 element vector specifying the time range.
;  RESOLUTION:  number of seconds resolution to return.
;  POLAR:       Computes polar coordinates if set.
;  PRE:		If set, use predicted (pre) data.
;  NODATA:	Returns 1 if no data available.
;  HEC:		If set, retrieve HEC data.
;  VAR_LAB:	Return polar data and use as var_labels in tplot.
;  GSM:		If set, retrieve GSM data.
;  LAT:		If set, retrieve LAT data.
;SEE ALSO: 
;  "make_cdf_index","loadcdf","loadcdfstr","loadallcdf"
;
;CREATED BY:	Davin Larson
;FILE:  load_wi_or.pro
;LAST MODIFICATION: 99/12/28
;-
function find_at_files,trange

if keyword_set(trange) then tr = time_double(trange) else $
	get_timespan,tr

if n_elements(tr) ne 2 then tr = [tr(0),tr(0)+24.*3600.]

defprebreak = time_double('1997-7-2')
defmast = 'wi_at_def_files'
premast = 'wi_at_pre_files'

if (tr[0] lt defprebreak) then begin
  if (tr[1] gt defprebreak) then begin
    deftrange = [tr[0],defprebreak]
    pretrange = [defprebreak,tr[1]]
    get_file_names,deffiles,time_range=deftrange,masterfile=defmast
    get_file_names,prefiles,time_range=pretrange,masterfile=premast
    return,[deffiles,prefiles]
  endif else begin
    get_file_names,deffiles,time_range=tr,masterfile=defmast
    return,deffiles
  endelse
endif else begin
  get_file_names,prefiles,time_range=tr,masterfile=premast
  return,prefiles
endelse
end

pro load_wi_at,time_range=trange,data=d,polar=polar,pre=pre,nodata=nodat,resolution=res,hec=hec,var_lab=var_lab,gsm=gsm,lat=lat


if keyword_set(pre) then begin
	masterfile='wi_at_pre_files'
	get_file_names,cdffilenames,masterfile=masterfile,time_range=trange
endif else cdffilenames=find_at_files(trange)
	
if (keyword_set(cdffilenames) eq 0) then begin
	print,'No WI_AT files for given time range.'
	return
endif

cdfnames = ['BODY_SPIN_RATE','GSE_R_ASCENSION','GSE_DECLINATION']

;if keyword_set(hec) then cdfnames = ['GSE_POS','HEC_POS']
;if keyword_set(gsm) then cdfnames = [cdfnames, 'GSM_POS']
;if keyword_set(lat) then cdfnames = [cdfnames,'LAT_SPACE']

d=0
nodat=0
loadallcdf,filenames=cdffilenames,cdfnames=cdfnames,data=d,time_range=trange,res=res

if keyword_set(d) eq 0 then begin
   if keyword_set(pre) then begin
      message,'No WIND orbit data during this time. Aborting.',/info
      nodat=1
      return
   endif else begin
      load_wi_at,time_range=trange,data=d,polar=polar,/pre,nodata=nodat,resolution=res,hec=hec,lat=lat
      return
   endelse
endif

;pos = dimen_shift(d.gse_pos/6370.,1)
;lab = ['X','Y','Z']


store_data,'wi_at_spin_rate',data={x:d.time,y:d.body_spin_rate/60}, min=-1e30
store_data,'wi_at_r_ascension',data={x:d.time,y:d.GSE_R_ASCENSION}, min=-1e30
store_data,'wi_at_declination',data={x:d.time,y:d.GSE_DECLINATION}, min=-1e30


end
